unit TExcellentExampleMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ShellAPI,
  Printers,
  PrnUtils,
  PrnForm,
  PrnDIB;


type
  TForm1 = class(TForm)
     Image1: TImage;
     PrintDialog1: TPrintDialog;
     ButtonPrintImage: TButton;
     ButtonPrintForm: TButton;
     ButtonGetDocumentation: TButton;
     ButtonBuyFormPrinter: TButton;
     ButtonBuyImagePrinter: TButton;
     GroupBoxOptions: TGroupBox;
     CheckBoxUseDDB: TCheckBox;
     CheckBox24BitFormOutput: TCheckBox;
     LabelSeeFaqForOtherOptionsLineOne: TLabel;
     LabelSeeFaqForOtherOptionsLineTwo: TLabel;
     LabelTryOfficialDemoApplication: TLabel;
     GroupBoxQuestionsOrProblems: TGroupBox;
     LabelEmail: TLabel;
     LabelMailTo: TLabel;
     LabelWebHomePage: TLabel;
     LabelWeb: TLabel;
     LabelTrialVersionNote: TLabel;
     GroupBoxReducePrintJobSize: TGroupBox;
     CheckBoxCompress: TCheckBox;
     CheckBoxWinStretch: TCheckBox;
     procedure FormCreate(Sender: TObject);
     procedure ButtonPrintFormClick(Sender: TObject);
     procedure ButtonPrintImageClick(Sender: TObject);
     procedure ButtonGetDocumentationClick(Sender: TObject);
     procedure ButtonBuyFormPrinterClick(Sender: TObject);
     procedure ButtonBuyImagePrinterClick(Sender: TObject);
     procedure LabelMailToClick(Sender: TObject);
     procedure LabelWebHomePageClick(Sender: TObject);
     procedure LabelTryOfficialDemoApplicationClick(Sender: TObject);
  private
      { Private declarations }
  public
    { Public declarations }
  end;


{A callback function for updating an abort dialog!}
function ImagePrinterCallbackFunction(UnitsDone : DWORD;
                                      TotalUnits : DWORD;
                                      AbortDialog : TAbortDialog) : BOOL; stdcall;


var
  Form1: TForm1;

implementation

{$R *.DFM}


procedure TForm1.FormCreate(Sender: TObject);
begin
 {Load test.bmp into the Image1 component}
  if (NOT FileExists('test.bmp')) then begin
    ShowMessage('Error - test.bmp not found in current directory!');
  end else begin
    Image1.Picture.Bitmap.LoadFromFile('test.bmp');
  end;  
end;


{Print the form scaled to a full page - with options!}
procedure TForm1.ButtonPrintFormClick(Sender: TObject);
var
  PrintOptions : TPrintFormData;
begin

 {Are we printing something right now?}
  if (Printer.Printing) then begin
    ShowMessage('Already printing... Please try again later!');
    exit;
  end;

 {Let the user select a printer}
  if (NOT PrintDialog1.Execute) then begin
    exit;
  end;

 {Prepare to set printing options}
  FillChar(PrintOptions, sizeof(PrintOptions), 0);
  PrintOptions.StrucSize := sizeof(PrintOptions);

 {Lets show a status dialog during the print job!}
  PrintOptions.UseAbortDialog := TRUE;

 {Lets show messages if there are problems printing the form}
  PrnFormSetDebugMsg(TRUE);

 {Allow the user to force the use 24 bit printing for problem printers}
  PrintOptions.Use24BitOutputOnly := CheckBox24BitFormOutput.Checked;

 {Allow user to force the use of DDBs for problem printers or speed up the print job}
  PrnFormSetDebugUseDDB(CheckBoxUseDDB.Checked);

  {Set print job size reduction factor}
  if (CheckBoxCompress.Checked) then begin
    PrnFormSetOutputScaleFactor(2, 2);
  end else begin
    PrnFormSetOutputScaleFactor(1, 1);
  end;

  {Set WinScale}
  if (CheckBoxWinStretch.Checked) then begin
    PrnFormSetOutputScaleFactor(1, 1);
    PrnFormSetDoWinScale(TRUE);
  end else begin
    PrnFormSetDoWinScale(FALSE);
  end;

 {Start the print job!}
  Printer.BeginDoc;

 {Print the form!}
  if (NOT PrintFormEx(self,
                      Printer.Canvas.Handle,
                      TRUE, {Center the form on the page}
                      TRUE, {Adjust for perfect margins}
                      @PrintOptions)) then begin
   {Abort the print job!}
    Printer.Abort;
    ShowMessage('User abort or other error!');
  end else begin
   {End the print job!}
    Printer.EndDoc;
  end;  

 {We are done!}
  ShowMessage('Printing Completed!');

end;



{Print test.bmp scaled to a full page - with abort dialog!}
procedure TForm1.ButtonPrintImageClick(Sender: TObject);
var
  AbortDialog : TAbortDialog;    {The abort dialog}
  BitmapInfo : PBITMAPINFO;      {A pointer to a BitmapInfo structure}
  Bits : pointer;                {A pointer to the bitmap bits}
  BitmapWidth : integer;         {The bitmap width}
  BitmapHeight : integer;        {The bitmap height}
begin

 {Are we printing something right now?}
  if (Printer.Printing) then begin
    ShowMessage('Already printing... Please try again later!');
    exit;
  end;

 {Do we have a bitmap to print?}
  if (NOT FileExists('test.bmp')) then begin
    ShowMessage('Error - test.bmp not found in current directory!');
    exit;
  end; 

 {Let the user select a printer}
  if (NOT PrintDialog1.Execute) then begin
    exit;
  end;

 {Load test.bmp as a Device Independent Bitmap}
  if (NOT LoadDIBFromFile('test.bmp',
                          pointer(BitmapInfo),
                          Bits,
                          BitmapWidth,
                          BitmapHeight)) then begin
      ShowMessage('Bitmap load error');
    exit;
  end;

 {Create the abort dialog}
  AbortDialog := CreateAbortDialog(Application.Handle, self);
  PrnDIBSetAbortDialogHandle(AbortDialog.Handle);

 {Lets show messages if there are problems printing the form}
  PrnDIBSetDebugMsg(TRUE);

 {Allow user to force the use of DDBs for problem printers or speed up the print job}
  PrnDIBSetDebugUseDDB(CheckBoxUseDDB.Checked);

  {Set print job size reduction factor}
  if (CheckBoxCompress.Checked) then begin
    PrnDIBSetOutputScaleFactor(2, 2);
  end else begin
    PrnDIBSetOutputScaleFactor(1, 1);
  end;

  {Set WinScale}
  if (CheckBoxWinStretch.Checked) then begin
    PrnDIBSetDoWinScale(TRUE);
    PrnDIBSetOutputScaleFactor(1, 1);
  end else begin
    PrnDIBSetDoWinScale(FALSE);
  end;

 {Start the print job!}
  Printer.BeginDoc;

 {Print the image!}
  if (PrintDIBitmapCB(Printer.Canvas.Handle,
                      BitmapInfo,
                      Bits,
                      TRUE,
                      TRUE,
                      @ImagePrinterCallbackFunction,
                      DWORD(AbortDialog)) < PRINT_SUCCESSFUL) then begin
   {Abort the print job!}
    Printer.Abort;
    ShowMessage('User abort or other error!');
  end else begin
   {End the print job!}
    Printer.EndDoc;
  end;

 {Clean Up!}
  FreeMemEx(BitmapInfo);
  FreeMemEx(Bits);
  FreeAbortDialog(AbortDialog);
  PrnDIBSetAbortDialogHandle(0);

 {We are done!}
  ShowMessage('Printing Completed!');

end;


{A callback function for updating an image printer abort dialog!}
function ImagePrinterCallbackFunction(UnitsDone : DWORD;
                                      TotalUnits : DWORD;
                                      AbortDialog : TAbortDialog) : BOOL; stdcall;
var
  s : string;
begin
 {Update the abort dialogs caption to give a status update}
  s := 'Printing - ' +
       IntToStr(Trunc(((UnitsDone / TotalUnits) * 100))) +
       '%';
  AbortDialogSetCaption(AbortDialog,
                        pChar(s));
 {Should we continue?}
  result := (NOT AbortDialogUserHasCanceled(AbortDialog));
end;



procedure TForm1.ButtonGetDocumentationClick(Sender: TObject);
begin
 {Open web browser and go to the TExcellent documentation page}
  ShellExecute(Form1.Handle,
               nil,
               'http://www.code4sale.com/joehecht/TExcellentDocs.htm',
               nil,
               nil,
               SW_SHOWNORMAL);
end;


procedure TForm1.ButtonBuyFormPrinterClick(Sender: TObject);
begin
 {Open web browser and go to the TExcellentFormPrinter BuyIt! page}
  ShellExecute(Form1.Handle,
               nil,
               'http://www.code4sale.com/order/texcellentformprinter.htm',
               nil,
               nil,
               SW_SHOWNORMAL);
end;


procedure TForm1.ButtonBuyImagePrinterClick(Sender: TObject);
begin
 {Open web browser and go to the TExcellentImagePrinter BuyIt! page}
  ShellExecute(Form1.Handle,
               nil,
               'http://www.code4sale.com/order/texcellentimageprinter.htm',
               nil,
               nil,
               SW_SHOWNORMAL);
end;

procedure TForm1.LabelMailToClick(Sender: TObject);
begin
 {Open the email application and send mail to sales@code4sale.com}
  ShellExecute(Application.Handle,
               'open',
               'mailto:sales@code4sale.com',
               nil,
               nil,
               SW_NORMAL);
end;


procedure TForm1.LabelWebHomePageClick(Sender: TObject);
begin
 {Open web browser and go to Joe Hecht Associates web page}
  ShellExecute(Form1.Handle,
               nil,
               'http://www.code4sale.com/joehecht/index.htm',
               nil,
               nil,
               SW_SHOWNORMAL);
end;

procedure TForm1.LabelTryOfficialDemoApplicationClick(Sender: TObject);
begin
 {Open TExcellentDemo program or try to download it}
  if (FileExists('TExcellentDemo.exe')) then begin
    WinExec('TExcellentDemo.exe',
            SW_SHOWNORMAL);
  end else begin
    ShellExecute(Form1.Handle,
                 nil,
                 'http://www.code4sale.com/joehecht/TExcellentDemo.zip',
                 nil,
                 nil,
                 SW_SHOWNORMAL);
  end;
end;


end.
